#!/usr/bin/perl

# Copyright 2009,2010,2011 Andreas Massouras
# Applies prinses variants to reference fasta file to obtain the "prinses consensus" fasta file

use strict;
use warnings;
push(@INC, $1) if ($0 =~ /^(.*)\//);
require 'variants.ph';

# Minimum depth for a variant to pass this filter:
my $minDepth = 3;

sub usage {
	print STDERR "Usage: $0 variants.verified.txt > variants.consensus.txt\n";
	exit 1;
}

usage() if (@ARGV != 1);
my $signature = scalar(localtime()) . " $0 " .join(" ", @ARGV);
print STDERR "$signature\n";

my ($variants, $header) = Variants::loadVariants(shift(@ARGV));
exit(0) if (! $variants);
my $count = 0;
my %count;
print "# $signature\n";
print $header;
foreach my $seqName (sort(keys(%$variants))) {
	my $ar = $variants->{$seqName};
	my $lastPos = 0;
	my @vars = sort { $a->[0] <=> $b->[0] } @$ar;
	my @filtered1 = ();

	# First filter out variants with decreasing read depth:
	foreach my $variant (sort { $a->[0] <=> $b->[0] } @$ar) {
		my ($position, $oldSeqLength, $newSeq, $strand, $text) = @$variant;

	# example: # read depth: before 3, after (21336307) 4 (upstream)
		next unless ($text =~ /read depth: before ([0-9\.]+), after \(\d+\) ([0-9\.]+)/);
		my ($before, $after)  = ($1, $2);
		next if ($after < $minDepth or $after < $before + 1);
	# example: # at first breakpoint: 0 -> 2.5, at second breakpoint: 1.5 -> 2.5
		if ($text =~ /at first breakpoint:\s*([0-9\.]+)\s*\-\>\s*([0-9\.]+)/) {
			my $bpBefore = $1;
			my $bpAfter = $2;
			next if ($bpAfter < $minDepth or $bpAfter < $bpBefore + 1);
		}
		if ($text =~ /at second breakpoint:\s*([0-9\.]+)\s*\-\>\s*([0-9\.]+)/) {
			my $bpBefore = $1;
			my $bpAfter = $2;
			next if ($bpAfter < $minDepth or $bpAfter < $bpBefore + 1);
		}
		push(@$variant, $after);	# add one element to avoid regexp match in the next loop
		push(@filtered1, $variant);
	}

	# Now remove overlapping variants
	my $i = 0;
	for (my $i = 0; $i < @filtered1;) {
		my ($position, $oldSeqLength, $newSeq, $strand, $text, $after) = @{$filtered1[$i]};

		my $j;
		my $nextAfter = 0;
		for ($j = $i + 1; $j < @filtered1; ++$j) {
			my ($nextPosition, $nextLength, $nextNewSeq, $nextStrand, $nextText, $na) = @{$filtered1[$j]};
			# check for overlapping variants:
			last if ($nextPosition >= $position + $oldSeqLength);
#			print STDERR "Overlapping $seqName:$position\n";
			$nextAfter = $na;
			last if ($after <= $nextAfter);
		}

		print $text if ($after > $nextAfter);
		$i = $j;
	}
}
